@@ -8,13 +8,15 @@
 # Copyright 2001,2009 Business Management Systems, Inc.
 # This code is under the GNU General Public License.  See COPYING for details.
 
-__version__ = '0.9.5'
+__version__ = '0.9.8'
 
 import os
+import re
 import milter
 import thread
 
 from milter import *
+from functools import wraps
 
 _seq_lock = thread.allocate_lock()
 _seq = 0
@@ -47,6 +49,9 @@ OPTIONAL_CALLBACKS = {
   'header':(P_NR_HDR,P_NOHDRS)
 }
 
+## @private
+R = re.compile(r'%+')
+
 ## @private
 def decode_mask(bits,names):
   t = [ (s,getattr(milter,s)) for s in names]
@@ -137,7 +142,12 @@ def nocallback(func):
   except KeyError:
     raise ValueError(
       '@nocallback applied to non-optional method: '+func.__name__)
-  return func
+  def wrapper(self,*args):
+    if func(self,*args) != CONTINUE:
+      raise RuntimeError('%s return code must be CONTINUE with @nocallback'
+        % func.__name__)
+    return CONTINUE
+  return wrapper
 
 ## Function decorator to disable callback reply.
 # If the MTA supports it, tells the MTA not to wait for a reply from
@@ -152,9 +162,14 @@ def noreply(func):
   except KeyError:
     raise ValueError(
       '@noreply applied to non-optional method: '+func.__name__)
+  @wraps(func)
   def wrapper(self,*args):
     rc = func(self,*args)
-    if self._protocol & nr_mask: return NOREPLY
+    if self._protocol & nr_mask:
+      if rc != CONTINUE:
+        raise RuntimeError('%s return code must be CONTINUE with @noreply'
+	  % func.__name__)
+      return NOREPLY
     return rc
   wrapper.milter_protocol = nr_mask
   return wrapper
@@ -212,7 +227,7 @@ class Base(object):
     # Some optional actions may be disabled by calling milter.set_flags(), or
     # by overriding the negotiate callback.  The bits include:
     # <code>ADDHDRS,CHGBODY,MODBODY,ADDRCPT,ADDRCPT_PAR,DELRCPT
-    #  CHGHDRS,QUARANTINE,CHGFROM,SETSMLIST</code>.
+    #  CHGHDRS,QUARANTINE,CHGFROM,SETSYMLIST</code>.
     # The <code>Milter.CURR_ACTS</code> bitmask is all actions
     # known when the milter module was compiled.
     # Application code can also inspect this field to determine
@@ -247,7 +262,9 @@ class Base(object):
 
   ## Defined by subclasses to write log messages.
   def log(self,*msg): pass
-  ## Called for each connection to the MTA.
+  ## Called for each connection to the MTA.  Called by the
+  # <a href="https://www.milter.org/developers/api/xxfi_connect">
+  # xxfi_connect</a> callback.  
   # The <code>hostname</code> provided by the local MTA is either
   # the PTR name or the IP in the form "[1.2.3.4]" if no PTR is available.
   # The format of hostaddr depends on the socket family:
@@ -260,6 +277,17 @@ class Base(object):
   # <dt><code>socket.AF_UNIX</code>
   # <dd>A string with the socketname
   # </dl>
+  # To vary behavior based on what port the client connected to,
+  # for example skipping blacklist checks for port 587 (which must 
+  # be authenticated), use @link #getsymval getsymval('{daemon_port}') @endlink.
+  # The <code>{daemon_port}</code> macro must be enabled in sendmail.cf
+  # <pre>
+  # O Milter.macros.connect=j, _, {daemon_name}, {daemon_port}, {if_name}, {if_addr}
+  # </pre>
+  # or sendmail.mc
+  # <pre>
+  # define(`confMILTER_MACROS_CONNECT', ``j, _, {daemon_name}, {daemon_port}, {if_name}, {if_addr}'')dnl
+  # </pre>
   # @param hostname the PTR name or bracketed IP of the SMTP client
   # @param family <code>socket.AF_INET</code>, <code>socket.AF_INET6</code>,
   #     or <code>socket.AF_UNIX</code>
@@ -271,12 +299,26 @@ class Base(object):
   # this almost always results in terminating the connection.
   @nocallback
   def hello(self,hostname): return CONTINUE
-  ## Called when the SMTP client says MAIL FROM.
+  ## Called when the SMTP client says MAIL FROM. Called by the
+  # <a href="https://www.milter.org/developers/api/xxfi_envfrom">
+  # xxfi_envfrom</a> callback.  
   # Returning REJECT rejects the message, but not the connection.
+  # The sender is the "envelope" from as defined by
+  # <a href="http://tools.ietf.org/html/rfc5321">RFC 5321</a>.
+  # For the From: header (author) defined in 
+  # <a href="http://tools.ietf.org/html/rfc5322">RFC 5322</a>,
+  # see @link #header the header callback @endlink.
   @nocallback
   def envfrom(self,f,*str): return CONTINUE
-  ## Called when the SMTP client says RCPT TO.
+  ## Called when the SMTP client says RCPT TO. Called by the
+  # <a href="https://www.milter.org/developers/api/xxfi_envrcpt">
+  # xxfi_envrcpt</a> callback.
   # Returning REJECT rejects the current recipient, not the entire message.
+  # The recipient is the "envelope" recipient as defined by 
+  # <a href="http://tools.ietf.org/html/rfc5321">RFC 5321</a>.
+  # For recipients defined in 
+  # <a href="http://tools.ietf.org/html/rfc5322">RFC 5322</a>, 
+  # for example To: or Cc:, see @link #header the header callback @endlink.
   @nocallback
   def envrcpt(self,to,*str): return CONTINUE
   ## Called when the SMTP client says DATA.
@@ -310,7 +352,7 @@ class Base(object):
   ## Called when the connection is closed.
   def close(self): return CONTINUE
 
-  ## Return mask of SMFIP_N.. protocol option bits to clear for this class
+  ## Return mask of SMFIP_N* protocol option bits to clear for this class
   # The @@nocallback and @@noreply decorators set the
   # <code>milter_protocol</code> function attribute to the protocol mask bit to
   # pass to libmilter, causing that callback or its reply to be skipped.
@@ -336,7 +378,10 @@ class Base(object):
     
   ## Negotiate milter protocol options.  Called by the
   # <a href="https://www.milter.org/developers/api/xxfi_negotiate">
-  # xffi_negotiate</a> callback.
+  # xffi_negotiate</a> callback.  This is an advanced callback,
+  # do not override unless you know what you are doing.  Most
+  # negotiation can be done simply by using the supplied 
+  # class and function decorators.
   # Options are passed as 
   # a list of 4 32-bit ints which can be modified and are passed
   # back to libmilter on return.
@@ -363,28 +408,59 @@ class Base(object):
   ## Return the value of an MTA macro.  Sendmail macro names
   # are either single chars (e.g. "j") or multiple chars enclosed
   # in braces (e.g. "{auth_type}").  Macro names are MTA dependent.
+  # See <a href="https://www.milter.org/developers/api/smfi_getsymval">
+  # smfi_getsymval</a> for default sendmail macros.
   # @param sym the macro name
   def getsymval(self,sym):
     return self._ctx.getsymval(sym)
 
   ## Set the SMTP reply code and message.
   # If the MTA does not support setmlreply, then only the
-  # first msg line is used.
+  # first msg line is used.  Any '%%' in a message line
+  # must be doubled, or libmilter will silently ignore the setreply.
+  # Beginning with 0.9.6, we test for that case and throw ValueError to avoid
+  # head scratching.  What will <i>really</i> irritate you, however,
+  # is that if you carefully double any '%%', your message will be
+  # sent - but with the '%%' still doubled!
+  # See <a href="https://www.milter.org/developers/api/smfi_setreply">
+  # smfi_setreply</a> for more information.
+  # @param rcode The three-digit (RFC 821/2821) SMTP reply code as a string. 
+  # rcode cannot be None, and <b>must be a valid 4XX or 5XX reply code</b>.
+  # @param xcode The extended (RFC 1893/2034) reply code. If xcode is None,
+  # no extended code is used. Otherwise, xcode must conform to RFC 1893/2034.
+  # @param msg The text part of the SMTP reply. If msg is None,
+  # an empty message is used.
+  # @param ml  Optional additional message lines.
   def setreply(self,rcode,xcode=None,msg=None,*ml):
+    for m in (msg,)+ml:
+      if 1 in [len(s)&1 for s in R.findall(m)]:
+        raise ValueError("'%' must be doubled: "+m)
     return self._ctx.setreply(rcode,xcode,msg,*ml)
 
   ## Tell the MTA which macro names will be used.
-  # The <code>Milter.SETSMLIST</code> action flag must be set.
+  # This information can reduce the size of messages received from sendmail,
+  # and hence could reduce bandwidth between sendmail and your milter where
+  # that is a factor.  The <code>Milter.SETSYMLIST</code> action flag must be
+  # set.  The protocol stages are M_CONNECT, M_HELO, M_ENVFROM, M_ENVRCPT,
+  # M_DATA, M_EOM, M_EOH.
   #
   # May only be called from negotiate callback.
-  # @since 0.9.2
-  # @param stage the protocol stage to set to macro list for
-  # @param macros a string with a space delimited list of macros
-  def setsmlist(self,stage,macros):
-    if not self._actions & SETSMLIST: raise DisabledAction("SETSMLIST")
-    if type(macros) in (list,tuple):
-      macros = ' '.join(macros)
-    return self._ctx.setsmlist(stage,macros)
+  # @since 0.9.8, previous version was misspelled!
+  # @param stage the protocol stage to set to macro list for, 
+  # one of the M_* constants defined in Milter
+  # @param macros space separated and/or lists of strings
+  def setsymlist(self,stage,*macros):
+    if not self._actions & SETSYMLIST: raise DisabledAction("SETSYMLIST")
+    a = []
+    for m in macros:
+      try:
+        m = m.encode('utf8')
+      except: pass
+      try:
+        m = m.split(' ')
+      except: pass
+      a += m
+    return self._ctx.setsmlist(stage,' '.join(a))
 
   # Milter methods which can only be called from eom callback.
 
@@ -714,4 +790,5 @@ for priv in ('os','milter','thread','factory','_seq','_seq_lock','__version__'):
 __all__ = __all__.keys()
 
 ## @example milter-template.py
+## @example milter-nomix.py
 #
